package org.ssssssss.example.mdjf.common;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * TODO desc
 *
 * @ClassName: CommonUtil
 * @author: hwang
 * @date: 2021/3/30 14:48
 */
public class CommonUtil {
  private static final Logger LOGGER = LogManager.getLogger(CommonUtil.class);

  /**
   * 字符串判空
   * @param str
   * @return
   */
  public static boolean isNullOrEmpty(String str)
  {
    if (str == null || "null".equals(str.toLowerCase().trim()) || "[null]".equals(str.toLowerCase().trim()) || "[]".equals(str.toLowerCase().trim()) || "{}".equals(str.toLowerCase().trim()))
    {
      return true;
    }

    return str.trim().isEmpty();
  }


  public static boolean isNotNullOrEmpty(String str)
  {
    if (str == null || "null".equals(str.toLowerCase().trim()) || "[null]".equals(str.toLowerCase().trim()) || "[]".equals(str.toLowerCase().trim()) || "{}".equals(str.toLowerCase().trim()))
    {
      return false;
    }

    return !str.trim().isEmpty();
  }


  /**
   * 经纬度计算距离  单位(米)
   * @param lng1
   * @param lat1
   * @param lng2
   * @param lat2
   * @return
   */
//  public static double getDistance2(Double lng1, Double lat1, Double lng2, Double lat2){
//    double earth=6371.393;
////        double earth=6371393;       // 米
//    double radLat1 = rad(lat1);
//    double radLat2 = rad(lat2);
//    double a = radLat1 - radLat2;
//    double b = rad(lng1) - rad(lng2);
//    double s = 2* Math.asin(Math.sqrt(Math.pow(Math.sin(a/2),2)+Math.cos(radLat1)*Math.cos(radLat2)*Math.pow(Math.sin(b/2),2)));
//    s=s*earth;
//    s=Math.round(s*10000d) / 10000d;
//    s=s*1000;
//    return s;
//  }
//
//  private static double rad(double d){
//    return d * Math.PI / 180.0;
//  }


  /**
   * 递归加载输入文件
   * @param filePath
   * @param filelist
   * @param preStr  文件前缀  contains(preStr)
   * @return
   */
  public static List<String> getFiles(String filePath, List<String> filelist ,String preStr) {
    File root = new File(filePath);
    if (root.exists()) {
      File[] files = root.listFiles();
      for (File file : files) {
        if (file.isDirectory()) {
          getFiles(file.getAbsolutePath(), filelist,preStr);
        } else {
          //文件前缀包含fzp的文件
          if (file.getName().contains(preStr)){
            File bak  = new File("bakPath/"+file.getName());
            if(bak.exists()){
              bak.delete();
            }
            file.renameTo(bak);   //拿到文件就放到备份目录  多此启动重复继续读
            filelist.add(bak.getAbsolutePath());
          }
        }
      }
    }
    return filelist;
  }





  /**
   * 写出JSON结果
   * @param file
   * @param tmpFileResult
   * @throws IOException
   */
  public static void writeJsonFile(File file,List<JSONObject> tmpFileResult) {
    LOGGER.info("开始写出文件");
    try (PrintWriter pw = new PrintWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"), true);){
      //如果没有文件就创建
      if (!file.isFile()) {
        file.createNewFile();
      }

      for (JSONObject tmpResult:tmpFileResult){
        pw.println(JSONObject.toJSONString(tmpResult, SerializerFeature.WriteMapNullValue));
      }
      LOGGER.info("成功写出文件");
    }catch (Exception e){
      LOGGER.error("{} 写出文件错误 {}, 异常信息： {}", Constant.SERVICE_NAME, file.getName(),e);
    }
  }

  /**
   * 读取文件解析为List<JSONObject>
   * @param file
   * @return List<JSONObject>
   */
  public static List<JSONObject> readFileJson(File file){
    List<JSONObject> reusltJson = new ArrayList<>();
    try (
        FileInputStream fis = new FileInputStream(file);
        InputStreamReader isr = new InputStreamReader(fis, "UTF-8");
        BufferedReader br = new BufferedReader(isr);
    ) {
      String line = null;
      while((line = br.readLine()) !=null && line.trim().length()>Constant.NUMBER_2){
        try {
          JSONObject  tmpJSON  = JSONObject.parseObject(line);
          reusltJson.add(tmpJSON);
        } catch (JSONException jsonE){
          LOGGER.error("{} 读取文件：{},错误内容： {}, JSON解析异常信息：{}", Constant.SERVICE_NAME,file.getName(),line ,jsonE);
        }
      }
      LOGGER.info("{} 读取输入文件成功： {} ", Constant.SERVICE_NAME,file.getName());
    } catch (IOException IoE) {//不需要自己关闭流
      LOGGER.error("{} 读取文件错误 {}, IO异常信息： {}", Constant.SERVICE_NAME, file.getName(),IoE);
    }
    return reusltJson;
  }




  /**
   * 获取当前时间的yyyyMMdd时间格式
   * @return
   */
  public static String getyyyyMMdd(){
    SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
    return formatter.format(System.currentTimeMillis());
  }


  /**
   * 将绝对描述转换为  年月日
   * @param str
   * @return
   */
  public static String TimesToYYMMDD(String str) {
    String result = null;
    Date date = new Date();
    date.setTime(Long.parseLong(str+"000"));
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
    try {
      result = sdf.format(date).substring(0, 8);
    } catch (Exception ex) {
      throw new  RuntimeException("绝对秒数存在异常");
    }
    return result;
  }


//
//
//  // TODO 均加final
//  private static  int corePoolSize = Constant.NUMBER_POOLSIZE;
//  private static  int maximumPoolSize = Constant.NUMBER_POOLSIZE;
//  private static  Long keepAliveTime = Long.valueOf(Constant.NUMBER_ALIVETIME);
//  private static  TimeUnit timeUnitSeconds = TimeUnit.MINUTES;
//  private static  BlockingQueue<Runnable> abQueue = new ArrayBlockingQueue<Runnable>(Constant.NUMBER_QUEUE_INIT);
//  private static  ThreadFactory threadFactory = Executors.defaultThreadFactory();
//  private static  RejectedExecutionHandler aPolicy = new ThreadPoolExecutor.AbortPolicy();
//  /**
//   * 核心线程数：corePoolSize
//   * 最大线程数：maximumPoolSize
//   * 线程空闲时间：keepAliveTime
//   * 有界队列长度：200
//   * 饱和策略：抛出异常
//   */
//  private static ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, keepAliveTime,
//      timeUnitSeconds, abQueue, threadFactory, aPolicy);
//  static {
//    threadPoolExecutor.prestartAllCoreThreads();
//  }

}
